@@ -1,5 +1,5 @@
 /***********************************************************************
- **
+ **F
  **   calculator.cpp
  **
  **   This file is part of Cumulus.
@@ -7,19 +7,19 @@
  ************************************************************************
  **
  **   Copyright (c): 2002      by André Somers
- **                  2008-2010 by Axel Pauli
+ **                  2008-2012 by Axel Pauli
  **
  **   This file is distributed under the terms of the General Public
  **   License. See the file COPYING for more information.
  **
- **   $Id: calculator.cpp 4617 2011-05-02 21:13:29Z axel $
+ **   $Id: calculator.cpp 4978 2012-03-07 12:14:49Z axel $
  **
  **   This class provides different calculator functionalities
  **
  ***********************************************************************/
 
-#include <stdlib.h>
 #include <cmath>
+#include <cstdlib>
 
 #include <QtGui>
 
@@ -48,6 +48,7 @@ Calculator::Calculator(QObject* parent) :
   QObject(parent),
   samplelist( LimitedList<FlightSample>( MAX_SAMPLECOUNT ) )
 {
+  setObjectName( "Calculator" );
   GeneralConfig *conf = GeneralConfig::instance();
 
   manualAltitude.setMeters( conf->getManualNavModeAltitude() );
@@ -139,10 +140,11 @@ const Altitude& Calculator::getAltimeterAltitude()
       return lastAGLAltitude; // AGL
       break;
     case 3:
-    default:
       return lastAHLAltitude; // AHL
       break;
     }
+
+  return lastAltitude;
 }
 
 const AltitudeCollection& Calculator::getAltitudeCollection()
@@ -246,22 +248,30 @@ void Calculator::slot_WaypointChange(Waypoint *newWp, bool userAction)
        selectedWp->taskPointIndex != -1 && newWp->taskPointIndex == -1 )
     {
       // A user action will overwrite a task point. That will stop the
-      // automatic task point switch. We will notice the user about
-      // that fact.
+      // automatic task point switch. We will notice the user about that fact.
+      QMessageBox mb( QMessageBox::Question,
+                      tr( "Replace current task point?" ),
+                      tr( "<html>"
+                          "A flight task is activated!<br>"
+                          "This selection will stop the automatic task point switch."
+                          "To avoid that make a selection from task menu."
+                          "<br>Do You really want to replace?"
+                          "</html>" ),
+                      QMessageBox::Yes | QMessageBox::No,
+                      QApplication::desktop() );
 
-      int answer=
-        QMessageBox::question( QApplication::activeWindow(),
-                               tr("Replace current task point?"),
-                               tr("<html>"
-                                  "A flight task is activated!<br>"
-                                  "This selection will stop the automatic task point switch."
-                                  "To avoid that make a selection from task menu."
-                                  "<br>Do You really want to replace?"
-                                  "</html>"),
-                               QMessageBox::Yes,
-                               QMessageBox::No | QMessageBox::Escape );
+      mb.setDefaultButton( QMessageBox::Yes );
 
-      if ( answer != QMessageBox::Yes )
+#ifdef ANDROID
+
+      mb.show();
+      QPoint pos = QApplication::desktop()->mapToGlobal( QPoint( QApplication::desktop()->width()/2 - mb.width()/2,
+                                                                 QApplication::desktop()->height()/2 - mb.height()/2 ) );
+      mb.move( pos );
+
+#endif
+
+      if ( mb.exec() != QMessageBox::Yes )
         {
           // do nothing change
           return;
@@ -405,7 +415,7 @@ void Calculator::calcDistance( bool autoWpSwitch )
   // considered in manual mode to make testing possible.
   bool inside = false;
 
-  if ( lastSpeed.getKph() > 35 || GpsNmea::gps->getGpsStatus() != GpsNmea::validFix )
+  if ( moving() || GpsNmea::gps->getGpsStatus() != GpsNmea::validFix )
     {
       inside = task->checkSector( curDistance, lastPosition, selectedWp->taskPointIndex );
     }
@@ -844,6 +854,9 @@ void Calculator::slot_changePosition(int direction)
           lastPosition = selectedWp->origP;
         }
       break;
+    case MapMatrix::Position:
+      // Nothing to do, new position was already set.
+      break;
     }
 
   // Check new position that it lays inside the earth coordinates +-90, +-180 degrees
@@ -894,6 +907,7 @@ void Calculator::slot_changePosition(int direction)
   calcDistance();
   calcBearing();
   calcGlidePath();
+
   // calculate always when moving manually (big delta !)
   _reachablelist->calculate(true);
   //qDebug("Elevation: %d m",_globalMapContents->findElevation(lastPosition) );
@@ -942,6 +956,14 @@ void Calculator::slot_changePositionWp()
 }
 
 
+/** Called if position was moved by using mouse. */
+void Calculator::slot_changePosition(QPoint& newPosition)
+{
+  lastPosition = newPosition;
+  slot_changePosition(MapMatrix::Position);
+}
+
+
 /** increment McCready value */
 void Calculator::slot_McUp()
 {
@@ -1128,7 +1150,7 @@ void Calculator::determineFlightStatus()
   /*
     WARNING: THIS CODE IS HIGHLY EXPERIMENTAL AND UNTESTED IN FLIGHT!
 
-    In principle, we are determining the flight status from the last 20
+    In principle, we are determining the flight status from the last 6
     seconds of flight. We may need to adjust this, as this is highly
     experimental code. However, if we already have a flight status, we
     only need to check if it has changed. If that is the case, we set
@@ -1144,12 +1166,10 @@ void Calculator::determineFlightStatus()
   */
 
   /*
-    We define a maximum turn rate of 10 degrees per second. This should
-    allow for course-corrections.  This can be a bit more advanced to
-    allow for temporary changes in circling speed in order to better
-    center a thermal.
+    We define a maximum deviation of 15 degrees from the cruising course.
+    This should be allowed for course corrections.
   */
-#define MAXCRUISEANGDIFF 8 //see above
+#define MAXCRUISEANGDIFF 15 //see above
 
   /*
     We define a minimum turn rate of 4 degrees per second (that is,
@@ -1161,12 +1181,12 @@ void Calculator::determineFlightStatus()
 #define MINTURNANGDIFF 4 //see above
 
   /*
-    We define a maximum altitude drift to refine the detection of a
+    We define a maximum altitude average drift to refine the detection of a
     standstill. Because GPS altitudes tend to drift, we can't rely
     on the exact same altitude over any length of time. We need to
     allow for some drift.
   */
-#define MAXALTDRIFT 10 //see above
+#define MAXALTDRIFTAVG 2 //see above
 
   /*
     We define an analysis time frame. This time frame determines how
@@ -1174,70 +1194,66 @@ void Calculator::determineFlightStatus()
     analysis. Note that the actual time difference between the first
     and the last sample used may differ from this time frame.
   */
-#define TIMEFRAME 10
+#define TIMEFRAME 6
 
-  if (samplelist.count() < 5)
-    return; //we need to have some samples in order to be able to analyze anything.
+  if( samplelist.count() < TIMEFRAME )
+    {
+      // We need to have some samples in order to be able to analyze anything.
+      return;
+    }
 
-  flightmode newFlightMode=unknown;
+  FlightMode newFlightMode = unknown;
 
-  //get headings from the last two samples
-  int lastHead=samplelist[0].vector.getAngleDeg();
-  int prevHead=samplelist[1].vector.getAngleDeg();
+  // get headings from the last two samples
+  int lastHead = samplelist[0].vector.getAngleDeg();
+  int prevHead = samplelist[1].vector.getAngleDeg();
 
-  //get the time difference between these samples
-  int timediff=samplelist[1].time.secsTo(samplelist[0].time);
+  // get the time difference between these samples
+  int timediff = samplelist[1].time.secsTo(samplelist[0].time);
 
-  if (timediff==0)
-    return; //if the time difference is 0, return (just to be sure). This will only cause problems...
-
-
-  //we are not doing a full analysis if we already have a flight mode. It suffices to check some
-  //basic criteria.
-  //lastFlightMode=unknown; //force complete evaluation for now.
-  //qDebug("Anglediff: %d",angleDiff(lastHead, prevHead));
+  if (timediff == 0)
+    {
+      // If the time difference is 0, return (just to be sure). This will only cause problems...
+      return;
+    }
 
+  // We are not doing a full analysis if we already have a flight mode.
+  // It suffices to check some basic criteria.
   switch (lastFlightMode)
     {
-    case standstill: //we are not moving at all!
-      if (samplelist[0].position == samplelist[1].position)    //may be too ridged, GPS errors could cause problems here
+    case standstill: // we are not moving at all!
+
+      if ( (samplelist[0].position == samplelist[1].position ||
+          ( dist(&samplelist[0].position, &samplelist[1].position) / double(timediff) ) < 0.005) &&
+           lastSpeed.getMps() <= 0.5 )
         {
-          return; //no change in flight mode
+          // may be too ridged, GPS errors could cause problems here
+          return;
         }
       else
         {
-          newFlightMode=unknown;
+          newFlightMode = unknown;
         }
       break;
 
-    case wave: //we are not moving at all, except vertically!  Needs lots of tweaking and testing...
-      if (samplelist[0].position == samplelist[1].position)    //may be too ridged, GPS errors could cause problems here
+    case cruising: // we are flying from point A to point B
+      if (abs(angleDiff(lastHead, _cruiseDirection)) <=  MAXCRUISEANGDIFF &&
+          samplelist[0].vector.getSpeed().getMps() > 0.5 )
         {
-          return; //no change in flight mslot_flightModeChangedode
+          return;
         }
       else
         {
-          newFlightMode=unknown;
-        }
-      break;
-
-    case cruising: //we are flying from point A to point B
-      if (abs(angleDiff(lastHead, _cruiseDirection)) <  MAXCRUISEANGDIFF &&
-          samplelist[0].vector.getSpeed().getMps()>5)
-        {
-          return; //no change in flight mode
-        }
-      else
-        {
-          newFlightMode=unknown;
+          // We left the cruising corridor. A new analysis has to be started.
+          newFlightMode = unknown;
           break;
         }
 
-    case circlingL:
+    case circlingR:
       //turning left means: the heading is decreasing
       if (angleDiff(prevHead, lastHead) > (-MINTURNANGDIFF * timediff))
         {
-          newFlightMode=unknown;
+          newFlightMode = unknown;
           break;
         }
       else
@@ -1245,12 +1261,11 @@ void Calculator::determineFlightStatus()
           return; //no change in flight mode
         }
 
-
-    case circlingR:
+    case circlingL:
       //turning right means: the heading is increasing
       if (angleDiff(prevHead, lastHead) < (MINTURNANGDIFF * timediff))
         {
-          newFlightMode=unknown;
+          newFlightMode = unknown;
           break;
         }
       else
@@ -1259,129 +1274,172 @@ void Calculator::determineFlightStatus()
         }
 
     default:
-      newFlightMode=unknown;
+      newFlightMode = unknown;
+      break;
     }
 
-  if (newFlightMode==unknown)
+  if ( newFlightMode == unknown )
     {
-      //we need some real analysis
-      QTime tmp= samplelist[0].time.addSecs(-TIMEFRAME); //reference time
-      int ls=1;
+      // qDebug() << "Flight mode unknown --> Start Analysis";
 
-      while ((samplelist[ls].time > tmp) && ( ls < samplelist.count()-1) )
-        ls++;
-      //ls now contains the index of the oldest sample we will use for this analysis.
-      //Newer samples have lower indices!
+      // we need some real analysis
+      QTime refTime = samplelist[0].time.addSecs(-TIMEFRAME);
+      int samples = 1;
+
+      while ((samplelist[samples].time > refTime) && ( samples < samplelist.count() - 1) )
+        {
+          samples++;
+        }
+
+      if( samples < 2 )
+        {
+          // At least we need 2 samples in our time window.
+          return;
+        }
+
+      // samples now contains the index of the oldest sample we will use for this analysis.
+      // Newer samples have lower indices!
 
       //initialize some values we will be needing...
-      bool mayBeL=true;     //this may be a left turn (that is, no big dir change to the right)
-      bool mayBeR=true;     //this may be a right turn (that is, no big dir change to the left)
-      int totalDirChange=0;  //total heading change. If cruising, this will be low, if turning, it will be high
-      int maxSpeed=0;        //maximum speed obtained in this set of samples
-      int aDiff=0;                 //difference in heading between two samples
-      int totalAltChange=0;  //total change in altitude (absolute)
-      int maxAltChange=0;  //maximum change of altitude between samples.
-      int altChange=0;
-      bool break_analysis=false; //flag to indicate we can stop further analysis.
+      bool mayBeL = true;     //this may be a left turn (that is, no big dir change to the right)
+      bool mayBeR = true;     //this may be a right turn (that is, no big dir change to the left)
+      int totalDirChange = 0; //total heading change. If cruising, this will be low, if turning, it will be high
+      double maxSpeed = 0.0;  //maximum speed obtained in this set of samples
+      int angDiff = 0;        //difference in heading between two samples
+      int totalAltChange = 0; //total change in altitude (absolute)
+      int maxAltChange = 0;   //maximum change of altitude between samples.
+      int altChange = 0;
+      bool break_analysis = false; //flag to indicate we can stop further analysis.
 
-
-      //loop through the samples to get some basic data we can use to distinguish flight modes
-      for (int i=ls-1;i>=0;i--)
+      // loop through the samples to get some basic data we can use to distinguish flight modes
+      for (int i = 0; i < samples; i++)
         {
-          aDiff = angleDiff( samplelist[i+1].vector.getAngleDeg(), samplelist[i].vector.getAngleDeg() );
-          //qDebug("analysis: angle1=%d, angle2=%d, diff=%d",int(samplelist->at(i+1)->vector.getAngleDeg()),int(samplelist->at(i)->vector.getAngleDeg()), aDiff);
-          //qDebug("analysis: position=(%d, %d)", samplelist->at(i)->position.x(),samplelist->at(i)->position.y() );
-          totalDirChange += abs(aDiff);
-          maxSpeed = qMax( maxSpeed, (int) rint(samplelist[i].vector.getSpeed().getKph()));
-          altChange = int(samplelist[i].altitude.getMeters() - samplelist[i+1].altitude.getMeters());
+          if( i < (samples - 1) )
+            {
+              // angDiff can be positive or negative according to the turn direction
+              angDiff = (int) rint(angleDiff( samplelist[i].vector.getAngleDeg(), samplelist[i+1].vector.getAngleDeg() ));
+
+              altChange = int( samplelist[i].altitude.getMeters() - samplelist[i+1].altitude.getMeters() );
+              //qDebug("analysis: position=(%d, %d)", samplelist->at(i)->position.x(),samplelist->at(i)->position.y() );
+            }
+          else
+            {
+              angDiff = 0;
+              altChange = 0;
+            }
+
+          // That is the average value of the direction change in degree.
+          // Can be positive or negation.
+          totalDirChange += angDiff;
+
+          maxSpeed = qMax( maxSpeed, samplelist[i].vector.getSpeed().getMps() );
           totalAltChange += altChange;
-          maxAltChange = int(qMax(abs(altChange), maxAltChange));
+          maxAltChange = qMax(abs(altChange), maxAltChange);
 
-          if (aDiff >  MINTURNANGDIFF)
-            mayBeL=false;
-          if (aDiff < -MINTURNANGDIFF)
-            mayBeR=false;
-          //qDebug("  analysis: sample: %d, speed: %f",i,samplelist->at(i)->vector.getKph());
+          if ( angDiff > MINTURNANGDIFF )
+            {
+              mayBeR = false;
+            }
+          else if ( angDiff < -MINTURNANGDIFF )
+            {
+              mayBeL = false;
+            }
+
+#if 0
+          qDebug("#Analysis(%d): angle1=%d, angle2=%d, angDiff=%d, speed=%f, alt1=%f, alt2=%f, altDiff=%d, tac=%d, tdc=%d, Vmax=%f",
+                 i,
+                 samplelist[i].vector.getAngleDeg(),
+                 (i < samples - 1) ? samplelist[i+1].vector.getAngleDeg() : 0,
+                 angDiff,
+                 samplelist[i].vector.getSpeed().getMps(),
+                 samplelist[i].altitude.getMeters(),
+                 (i < samples - 1) ? samplelist[i+1].altitude.getMeters() : 0,
+                 altChange,
+                 totalAltChange,
+                 totalDirChange,
+                 maxSpeed);
+#endif
         }
-      //qDebug ("analysis: aDiff=%d, totalDirChange=%d, maxSpeed=%d, totalAltChange=%d, maxAltChange=%d, maybeLeft=%d, maybeRight=%d",aDiff, totalDirChange, maxSpeed, totalAltChange, maxAltChange, int(mayBeL), int(mayBeR));
 
-      //try standstill. We are using a value > 0 because of possible GPS errors.
+      // try standstill. We are using a value > 0 because of possible GPS errors.
       /*
         The detection of stand stills may be extended further by checking if the altitude matches the terrain altitude. If not
         (or no where near), we can not assume a standstill. This is probably wave flying.
       */
-      if (maxSpeed<10)   // if we get under 10 kph for maximum speed, we may be standing still
+      if ( maxSpeed < 0.5 ) // if we get under 0.5m/s for maximum speed, we may be standing still
         {
-          if (abs(totalAltChange) * 2 <= MAXALTDRIFT && maxAltChange <= MAXALTDRIFT)    //check if we had any significant altitude changes
+          // check if we had any significant altitude changes
+          if ( (abs(totalAltChange) / (samples-1)) <= MAXALTDRIFTAVG )
             {
-              newFlightMode=standstill;
-              break_analysis=true;
-            }
-          else
-            {
-              newFlightMode=wave;
-              break_analysis=true;
+              newFlightMode = standstill;
+              break_analysis = true;
+              // qDebug() << "-->StandStill";
             }
         }
 
       if (!break_analysis)
         {
-          //get the time difference between the first and the last sample. This might not be the 20 secs we were planning to use at all!
-          timediff= samplelist[ls-1].time.secsTo(samplelist[0].time);
+          // Get the time difference between the first and the last sample.
+          // This might not be the 20 secs we were planning to use at all!
+          timediff = samplelist[samples-1].time.secsTo(samplelist[0].time);
 
-          //see if we might be cruising...
-          if (mayBeL && mayBeR)   //basicly, we have been going (almost) strait it seems...
-            {
-              if (totalDirChange < 2 * timediff)
-                {
-                  //qDebug("analysis: distance=%f m, time difference=%d s",dist(&samplelist->at(ls-1)->position,&samplelist[0].position)*1000,timediff);
-                  if (dist(&samplelist[ls-1].position, &samplelist[0].position)*1000 > 5*timediff) //our average speed should be at least 5 m/s to qualify for cruising
-                    newFlightMode=cruising;
-                  _cruiseDirection=samplelist[0].vector.getAngleDeg();
-                  // qDebug("Cruise direction: %d.",_cruiseDirection);
-                }
-              break_analysis=true;
-            }
-        }
-
-      if (!break_analysis)
-        {
-          //So, we are not standing still, nor are we cruising. Circling then maybe?
-          if ((MINTURNANGDIFF * timediff ) < totalDirChange)   //if circling, we should have a minimum average turnrate of MINTURNANGDIFF degrees per second
+          // So, we are not standing still, nor are we cruising. Circling then maybe?
+          if ( abs(totalDirChange) > (MINTURNANGDIFF * timediff) )
             {
+              // if circling, we should have a minimum average turn rate of
+              // MINTURNANGDIFF degrees per second
               if (mayBeL && !mayBeR)
-                newFlightMode=circlingL;
+                {
+                  newFlightMode = circlingL;
+                  break_analysis = true;
+                  // qDebug() << "-->circlingL";
+                }
+
               if (mayBeR && !mayBeL)
-                newFlightMode=circlingR;
-              //we have a new mode: we are circling! If neither of the above are true, we are doing something
-              //funny, and the flight mode 'unknown' still applies, so we may also quit analyzing...
+                {
+                  newFlightMode = circlingR;
+                  break_analysis = true;
+                  // qDebug() << "-->circlingR";
+                }
+
+              // we have a new mode: we are circling! If neither of the above are true, we are doing something
+              // funny, and the flight mode 'unknown' still applies, so we may also quit analyzing...
             }
         }
+
+      if ( !break_analysis )
+        {
+          // If all other modes are not true we assume the cruise mode.
+          newFlightMode = cruising;
+          break_analysis = true;
+
+          // save current heading for cruise check.
+          _cruiseDirection = samplelist[0].vector.getAngleDeg();
+          // qDebug("-->Cruise direction: %d.", _cruiseDirection);
+        }
     }
 
-  if (newFlightMode!=lastFlightMode)
+  if (newFlightMode != lastFlightMode)
     {
-      lastFlightMode=newFlightMode;
-      samplelist[0].marker=++_marker;
-      // qDebug("new flightmode: %d",lastFlightMode);
-      slot_flightModeChanged(newFlightMode);
+      lastFlightMode = newFlightMode;
+      samplelist[0].marker = ++_marker;
+      flightModeChanged( newFlightMode );
+      // qDebug( "new FlightMode: %d", lastFlightMode );
     }
 }
 
-
 /** Called if the status of the GPS changes. */
 void Calculator::slot_GpsStatus(GpsNmea::GpsStatus newState)
 {
   // qDebug("connection status changed...");
-  flightmode newFlightMode=unknown;
-  // qDebug("new flightmode=%d, last flightmode=%d",newFlightMode, lastFlightMode);
+  FlightMode newFlightMode = unknown;
 
-  if (newFlightMode!=lastFlightMode)
+  if (newFlightMode != lastFlightMode)
     {
-      lastFlightMode=newFlightMode;
+      lastFlightMode = newFlightMode;
       samplelist[0].marker=++_marker;
-      // qDebug("new flightmode: %d",lastFlightMode);
-      slot_flightModeChanged(newFlightMode);
+      // qDebug("new FlightMode: %d",lastFlightMode);
+      flightModeChanged(newFlightMode);
     }
 
   if ( newState == GpsNmea::noFix )
@@ -1393,9 +1451,8 @@ void Calculator::slot_GpsStatus(GpsNmea::GpsStatus newState)
     }
 }
 
-
-/** This slot is used internally to re-emit the flight mode signal with the marker value */
-void Calculator::slot_flightModeChanged(Calculator::flightmode fm)
+/** This function is used internally to emit the flight mode signal with the marker value */
+void Calculator::flightModeChanded(Calculator::FlightMode fm)
 {
   if ( _calculateWind )
     {
@@ -1477,8 +1534,6 @@ bool Calculator::matchesFlightMode(GeneralConfig::UseInMode mode)
     return lastFlightMode == standstill;
   if (mode==GeneralConfig::cruising)
     return lastFlightMode == cruising;
-  if (mode==GeneralConfig::wave)
-    return lastFlightMode == wave;
   if (mode==GeneralConfig::circling)
     return (lastFlightMode == circlingL || lastFlightMode == circlingR);
 
@@ -1594,21 +1649,29 @@ void Calculator::slot_startTask()
         if( selectedWp->taskPointIndex != -1 &&
             selectedWp->taskPointIndex != tp2Taken->taskPointIndex )
           {
-            int answer =
-              QMessageBox::question( QApplication::activeWindow(),
-                                     tr("Restart current task?"),
-                                     tr("<html>"
-                                        "A flight task is running!<br>"
-                                        "This command will start the<br>"
-                                        "task again at the beginning."
-                                        "<br>Do You really want to restart?"
-                                        "</html>"),
-                                     QMessageBox::Yes,
-                                     QMessageBox::No | QMessageBox::Escape );
+            QMessageBox mb( QMessageBox::Question,
+                            tr( "Restart current task?" ),
+                            tr( "<html>"
+                                "A flight task is running!<br>"
+                                "This command will start the<br>"
+                                "task again at the beginning."
+                                "<br>Do You really want to restart?"
+                                "</html>" ),
+                            QMessageBox::Yes | QMessageBox::No,
+                            QApplication::desktop() );
 
-            if ( answer != QMessageBox::Yes )
+            mb.setDefaultButton( QMessageBox::Yes );
+
+#ifdef ANDROID
+
+            mb.show();
+            QPoint pos = QApplication::desktop()->mapToGlobal( QPoint( QApplication::desktop()->width()/2 - mb.width()/2,
+                                                                            QApplication::desktop()->height()/2 - mb.height()/2 ) );
+            mb.move( pos );
+
+#endif
+            if ( mb.exec() == QMessageBox::No )
               {
-                // do ignore the request
                 return;
               }
           }
@@ -1616,3 +1679,32 @@ void Calculator::slot_startTask()
 
     slot_WaypointChange( tp2Taken, true );
 }
+
+/**
+ * @returns true if we are faster in move as or equal 35km/h.
+ */
+bool Calculator::moving()
+{
+  const double Limit = 35000.0 / 3600.0; // 35Km/h as m/s
+
+  if( samplelist.count() < 5 )
+    {
+      // We need to have some samples in order to be able to analyze speed.
+      return false;
+    }
+
+  double speed = 0.0;
+
+  for( int i = 0; i < 5; i++ )
+    {
+      // average speed about 5s
+      speed += samplelist[i].vector.getSpeed().getMps();
+    }
+
+  if( (speed / 5.0) >= Limit )
+    {
+      return true;
+    }
+
+  return false;
+}
