@@ -7,12 +7,12 @@
 ************************************************************************
 **
 **   Copyright (c):  2002      by André Somers
-**                   2007-2010 by Axel Pauli
+**                   2007-2012 by Axel Pauli
 **
 **   This file is distributed under the terms of the General Public
 **   License. See the file COPYING for more information.
 **
-**   $Id: configwidget.h 4502 2010-12-09 22:32:02Z axel $
+**   $Id: configwidget.h 5018 2012-03-22 14:19:26Z axel $
 **
 ***********************************************************************/
 
@@ -25,16 +25,25 @@
  *
  * This is the general configuration widget for Cumulus.
  *
- * \date 2002-2010
+ * \date 2002-2012
+ *
+ * \version $Id: configwidget.h 5018 2012-03-22 14:19:26Z axel $
  */
 
 #ifndef _ConfigWidget_h
 #define _ConfigWidget_h
 
+#include <QWidget>
 #include <QStringList>
 
 #include "settingspagepersonal.h"
+
+#ifdef ANDROID
+#include "settingspagegps4a.h"
+#else
 #include "settingspagegps.h"
+#endif
+
 #include "settingspageunits.h"
 #include "settingspagemapobjects.h"
 #include "settingspagemapsettings.h"
@@ -46,6 +55,9 @@
 #include "settingspagelooknfeel.h"
 #include "settingspageterraincolors.h"
 
+class QSize;
+class QTabWidget;
+
 class ConfigWidget : public QWidget
 {
   Q_OBJECT
@@ -66,6 +78,10 @@ public:
    */
   virtual ~ConfigWidget();
 
+protected:
+
+  bool eventFilter( QObject *o , QEvent *e );
+
 public slots:
 
   /**
@@ -135,11 +151,20 @@ signals:
 
 private:
 
+  // Overall widget
+  QTabWidget* m_tabWidget;
+
   // Single configuration widgets
   SettingsPagePersonal*      spp;
   SettingsPageGlider*        spgl;
   SettingsPageTask*          spt;
+
+#ifdef ANDROID
+  SettingsPageGPS4A*         spg;
+#else
   SettingsPageGPS*           spg;
+#endif
+
   SettingsPageMapSettings*   spms;
   SettingsPageTerrainColors* sptc;
   SettingsPageMapObjects*    spmo;
@@ -150,6 +175,11 @@ private:
   SettingsPageLookNFeel*     splnf;
 
   bool loadConfig; // controls loading of configuration data
+
+#ifdef ANDROID
+  QSize fullSize;
+#endif
+
 };
 
 #endif
