@@ -7,12 +7,12 @@
  ************************************************************************
  **
  **   Copyright (c):  2004      by André Somers
- **                   2007-2011 by Axel Pauli
+ **                   2007-2012 by Axel Pauli
  **
  **   This file is distributed under the terms of the General Public
  **   License. See the file COPYING for more information.
  **
- **   $Id: generalconfig.cpp 4726 2011-12-20 15:05:02Z axel $
+ **   $Id: generalconfig.cpp 4894 2012-01-25 20:59:33Z axel $
  **
  ***********************************************************************/
 
@@ -37,6 +37,10 @@ using namespace std;
 #include "maemostyle.h"
 #endif
 
+#ifdef ANDROID
+#include "androidstyle.h"
+#endif
+
 // define NULL static instance
 GeneralConfig* GeneralConfig::_theInstance = 0;
 
@@ -74,7 +78,7 @@ void GeneralConfig::load()
 {
   // cumulus main data
   beginGroup("Main");
-  _installRoot = value("InstallRoot", "./").toString();
+  _appRoot = value("InstallRoot", "./").toString();
   _builtDate = value("BuiltDate", __DATE__).toString();
   endGroup();
 
@@ -89,10 +93,14 @@ void GeneralConfig::load()
 
   _mapSideFrameColor     = QColor( value("MapSideFrameColor", "#687ec6").toString() );
 
-#ifndef MAEMO
+#ifdef MAEMO
   _guiStyle              = value("Style", "Plastique").toString();
+#else
+#ifdef ANDROID
+  _guiStyle              = value("Style", "Android").toString();
 #else
   _guiStyle              = value("Style", "Plastique").toString();
+#endif
 #endif
 
   _guiFont               = value("Font", "").toString();
@@ -115,7 +123,7 @@ void GeneralConfig::load()
   _forceDrawingDistance = value("forceLowAirspaceDrawingDistance", 150.0).toDouble();
   _airspaceFileList     = value("FileList", QStringList(QString("All"))).toStringList();
 
-#ifndef MAEMO
+#if ! defined (MAEMO) && ! defined (ANDROID)
   _airspaceLineWidth = value( "AirSpaceLineWidth", 5 ).toInt();
 #else
   _airspaceLineWidth = value( "AirSpaceLineWidth", 7 ).toInt();
@@ -243,7 +251,7 @@ void GeneralConfig::load()
   endGroup();
 
   beginGroup("Task");
-#ifndef MAEMO
+#if ! defined (MAEMO) && ! defined (ANDROID)
   _targetLineWidth = value( "TargetLineWidth", 5 ).toInt();
 #else
   _targetLineWidth = value( "TargetLineWidth", 7 ).toInt();
@@ -457,7 +465,7 @@ void GeneralConfig::save()
 {
   // cumulus main data
   beginGroup("Main");
-  setValue("InstallRoot", _installRoot );
+  setValue("InstallRoot", _appRoot );
   setValue("BuiltDate", _builtDate);
   endGroup();
 
@@ -1080,7 +1088,7 @@ void GeneralConfig::setGpsStartClientOption(const bool newValue)
 /** Gets the Gps Client path */
 QString GeneralConfig::getGpsClientPath()
 {
-  return _installRoot + "/bin/";
+  return _appRoot + "/bin/";
 }
 
 
@@ -1327,7 +1335,7 @@ QPixmap GeneralConfig::loadPixmap( const QString& pixmapName )
     }
 
   // determine absolute path to pixmap directory
-  QString path( _installRoot + "/icons/" + pixmapName );
+  QString path( _appRoot + "/icons/" + pixmapName );
 
   QPixmap pm;
 
@@ -1360,7 +1368,7 @@ QPixmap GeneralConfig::loadPixmap( const QString& pixmapName )
 void GeneralConfig::removePixmap( const QString& pixmapName )
 {
   // determine absolute path to pixmap directory and remove pixmap
-  QPixmapCache::remove( _installRoot + "/icons/" + pixmapName );
+  QPixmapCache::remove( _appRoot + "/icons/" + pixmapName );
 }
 
 /**
@@ -1462,7 +1470,10 @@ QString GeneralConfig::getUserDefaultRootDir()
 
 #endif
 
-  root += "/Cumulus"; // Cumulus default root directory
+  // Note under Android $HOME is set to the root of user's external storage directory
+#ifndef ANDROID
+  root += "/Cumulus"; // Cumulus default user root directory
+#endif
 
   return root;
 }
@@ -1549,6 +1560,8 @@ void GeneralConfig::setUserDataDirectory( QString newDir )
 /** Get the GPS default device depending on the hardware type. */
 QString GeneralConfig::getGpsDefaultDevice()
 {
+#ifdef MAEMO
+
   if( HwInfo::instance()->getSubType() == HwInfo::n800 )
     {
       return MAEMO_LOCATION_SERVICE;
@@ -1563,6 +1576,7 @@ QString GeneralConfig::getGpsDefaultDevice()
     {
       return MAEMO_LOCATION_SERVICE;
     }
+#endif
 
   // Default in unknown case is the serial device
   return "/dev/ttyS0";
@@ -1600,6 +1614,12 @@ void GeneralConfig::setOurGuiStyle()
 {
   qDebug() << "Setting GuiStyle:" << _guiStyle;
 
+#ifdef ANDROID
+
+  QStyle* style = QApplication::setStyle( _guiStyle );
+  QApplication::setStyle( new AndroidProxyStyle( style ) );
+
+#else
 #ifdef MAEMO
 
   QStyle* style = QApplication::setStyle( _guiStyle );
@@ -1609,6 +1629,7 @@ void GeneralConfig::setOurGuiStyle()
 
   QApplication::setStyle( _guiStyle );
 
+#endif
 #endif
 }
 
@@ -1825,7 +1846,7 @@ void GeneralConfig::setLanguage( const QString& newValue )
   if( ! _language.isEmpty() )
     {
       QString langFile = "cumulus_" + _language + ".qm";
-      QString langDir = _installRoot + "/locale/" + _language;
+      QString langDir = _dataRoot + "/locale/" + _language;
 
       // Load GUI translation file
       if( ! cumulusTranslator )
