@@ -12,7 +12,7 @@
 **   This file is distributed under the terms of the General Public
 **   License. See the file COPYING for more information.
 **
-**   $Id: waypointlistwidget.cpp 4617 2011-05-02 21:13:29Z axel $
+**   $Id: waypointlistwidget.cpp 4710 2011-11-25 14:32:36Z axel $
 **
 ***********************************************************************/
 
@@ -169,6 +169,22 @@ void WaypointListWidget::deleteSelectedWaypoints()
     }
 }
 
+/**
+ * Removes all waypoints from the list.
+ */
+void WaypointListWidget::deleteAllWaypoints()
+{
+  // remove all waypoints in the catalog
+  _globalMapContents->getWaypointList().clear();
+
+  // save the modified catalog
+  _globalMapContents->saveWaypointList();
+
+  list->clear();
+  filter->reset();
+  resizeListColumns();
+}
+
 // JD: after adding, deleting or name-changing a waypoint the filter
 // and the view must always be reset to regain consistency
 
